'use strict';

const AWS = require('aws-sdk');
const kinesis = new AWS.Kinesis({region: 'us-west-2'});

const agg = require('aws-kinesis-agg');
const RecordAggregator = require('aws-kinesis-agg/RecordAggregator');
// const aggregate = RecordAggregator.aggregate;

const uuid = require('uuid');

const Strings = require('core-functions/strings');
const stringify = Strings.stringify;

const Promises = require('core-functions/promises');

const tries = require('core-functions/tries');
// const Success = tries.Success;
const Failure = tries.Failure;

const kinesisPromises = [];
const samples = require('../samples');

const messages = [{a: 'ABC'}, {b: 'DEF'}]; //, JSON.stringify({b:'DEF'}), JSON.stringify({c:'GHI'})];
const ids = messages.map(() => uuid.v4());
const rawRecords = messages.map((data, i) => {
  const record = samples.sampleKinesisPutRecordRequest('TEST_Stream', ids[i], data);
  console.log(`########## record[${i}] = ${JSON.stringify(record, undefined, 2)}`);
  return record;
});
//const event = samples.sampleKinesisEventWithRecords(rawRecords);

// //Send simple putRecord requests
// const simpleKinesisPromises = rawRecords.map(putRecordRequest => {
//   return kinesis.putRecord(putRecordRequest).promise();
// });
// Promises.every(simpleKinesisPromises).then(
//   outcomes => {
//     console.log(`##### simple Kinesis ${outcomes.length} outcomes = [${outcomes.map(o => o.toString()).join(', ')}]`);
//     const successes = outcomes.filter(o => o.isSuccess());
//     console.log(`##### simple Kinesis successes = ${successes.length}`);
//     const failures = outcomes.filter(o => o.isFailure());
//     console.log(`##### simple Kinesis failures = ${failures.length}`);
//   },
//   err => {
//     console.log(`##### simple Kinesis failed`, err);
//   }
// );


function onReady(err, encoded) {
  console.log(`################ onReady aggregator = ${stringify(aggregator)}`);
  if (err) {
    console.error(`Failure passed to onReady`, err);
    // console.log("Encoded records of size " + Buffer.byteLength(encoded) + " received");
    kinesisPromises.push(Promise.resolve(new Failure(err)));
    return;
  }
  console.log(`onReady called with encoded (${stringify(encoded)})`);
  // const request = {
  //   StreamName: 'TEST_Stream',
  //   PartitionKey: `${Math.random()}`,
  //   Data: encoded.toString('base64'),
  //   // ExplicitHashKey: ...
  //   // SequenceNumberForOrdering: ...
  // };

  // MANUALLY ADD MISSING StreamName!!!!
  encoded.StreamName = 'TEST_Stream';

  // kinesis.putRecord(Math.random(), encoded.toString('base64'));
  const promise = Promises.one(kinesis.putRecord(encoded).promise());
  console.log(`onReady promise = ${stringify(promise)}`);
  promise.then(
    outcome => {
      console.log(`onReady promise resolved with ${outcome}`);
    },
    err => {
      console.log(`onReady promise rejected with ${err}`);
    }
  );
  kinesisPromises.push(promise);
  Promises.every(kinesisPromises).then(
    outcomes => {
      console.log(`##### onReady Kinesis ${outcomes.length} outcomes = [${outcomes.map(o => o.toString()).join(', ')}]`);
      const successes = outcomes.filter(o => o.isSuccess());
      console.log(`##### onReady Kinesis successes = ${successes.length}`);
      const failures = outcomes.filter(o => o.isFailure());
      console.log(`##### onReady Kinesis failures = ${failures.length}`);
    }
  );

}

function afterRecords() {
  console.log(`################ afterRecords aggregator = ${stringify(aggregator)}`);
  const n = arguments.length;
  let args = new Array(n);
  for (let i = 0; i < n; ++i) {
    args[i] = arguments[i];
  }
  console.log(`afterRecords ${n} arguments = ${stringify(args)}`);
  Promises.every(kinesisPromises).then(
    outcomes => {
      console.log(`##### afterRecords Kinesis outcomes = [${outcomes.map(o => o.toString()).join(', ')}]`);
      const successes = outcomes.filter(o => o.isSuccess());
      console.log(`##### afterRecords Kinesis successes = ${successes.length}`);
      const failures = outcomes.filter(o => o.isFailure());
      console.log(`##### afterRecords Kinesis failures = ${failures.length}`);
    }
  );
}


// const RecordAggregator = agg.RecordAggregator;
const aggregator = new RecordAggregator();
console.log(`################ aggregator = ${stringify(aggregator)}`);
console.log(`################ RecordAggregator = ${stringify(RecordAggregator)}`);

// aggregator.aggregateRecords()
// aggregator.aggregate(rawRecords, onReady);
// const kinesisObjects = rawRecords.map((r, i) => {
//   const kinesisObject = r.kinesis;
//   console.log(`########## kinesisObject[${i}] = ${JSON.stringify(kinesisObject, undefined, 2)}`);
//   return kinesisObject;
// });
aggregator.aggregateRecords(rawRecords, false, afterRecords, onReady);
// RecordAggregator.aggregate(kinesisObjects, onReady);
// const aggregate = Promises.wrap(aggregator.aggregate);
aggregator.flushBufferedRecords(onReady);
const flushBufferedRecords = Promises.wrap(aggregator.flushBufferedRecords);

flushBufferedRecords.call(aggregator).then(
  results => {
    console.log(`Flushed buffered records - results ${JSON.stringify(results)}`);
    return onReady(null, results);
    // console.log(`kinesisPromises = ${stringify(kinesisPromises)}`);
    // return Promises.every(kinesisPromises).then(
    //   outcomes => {
    //     const successes = outcomes.filter(o => o.isSuccess());
    //     console.log(`Kinesis successes = ${successes.length}`);
    //     const failures = outcomes.filter(o => o.isFailure());
    //     console.log(`Kinesis failures = ${failures.length}`);
    //   }
    // );
  },
  err => {
    console.error(`Failed to flush buffered records`, err);
  }
);

// const aggregatedPromise = aggregate(rawRecords).then(
//   encoded => putKinesisRecord(encoded),
//   err => {
//     console.error(`Failed to aggregate ${rawRecords}`, err);
//     throw err;
//   }
// );
//
// aggregatedPromise.then(
//   results => {
//     console.log(`aggregate results`);
//
//
//     return flushBufferedRecords()
//   },
//   err => {
//     console.error(`aggregate results failed`, err);
//     throw err;
//   }
// );







